/*********************************************************************NVMH3****
File: $Id: //sw/devtools/ShaderLibrary/main/HLSL/checker3d.fx#1 $

Copyright NVIDIA Corporation 2007
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY
LOSS) ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF
NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

% 3D Checkerboard effect, created by procedural texturing.
% Texture is pre-calculated, using the HLSL virtual machine (VM).
% To see a purely analytic alternative that gives good anti-aliasing
% at all scales, see "checker3d_math.fx"
% As an "extra," the check pattern is also applied to the specular
% value, to make the variation between materials stronger.

keywords: material pattern virtual_machine

The checker pattern is aligned to world coordinates in this sample.

$Date: 2007/06/15 $
******************************************************************************/
//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 matWorldInv;
float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matViewInv;

#ifndef _H_STRIPE_TEX_
#define _H_STRIPE_TEX_

// caller-override-able

#ifndef STRIPE_TEX_SIZE
#define STRIPE_TEX_SIZE 128
#endif /* STRIPE_TEX_SIZE */

#ifndef DEFAULT_BALANCE
#define DEFAULT_BALANCE (0.5)
#endif /* DEFAULT_BALANCE */

/************************************************************/
/*** TWEAKABLES *********************************************/
/************************************************************/

#define DECLARE_BALANCE float Balance = DEFAULT_BALANCE;

/////////////// prodecural texture /////////////

/*********** texture shader ******/

float4 make_stripe_tex(float2 Pos : POSITION,float ps : PSIZE) : COLOR
{
   float v = 0;
   float nx = Pos.x+ps; // keep the last column full-on, always
   v = nx > Pos.y;
   return float4(v.xxxx);
}

// texture declaration

texture mtlSkin1;

sampler2D StripeSampler = sampler_state {
    Texture = <mtlSkin1>;
    MinFilter = Linear;
    MipFilter = Linear;
    MagFilter = Linear;
    AddressU = Wrap;
    AddressV = Clamp;
};

////////////////////////////////////////////
// Utility Functions ///////////////////////
////////////////////////////////////////////

// base function: "Balance" is in W term
float stripe(float4 XYZW) { return tex2D(StripeSampler,XYZW.xw).x; }

float stripe(float4 XYZW,float Balance) {
    return stripe(float4(XYZW.xyz,Balance)); }

float stripe(float3 XYZ,float Balance) {
    return stripe(float4(XYZ.xyz,Balance)); }

float stripe(float2 XY,float Balance) {
    return stripe(float4(XY.xyy,Balance)); }

float stripe(float X,float Balance) {
    return stripe(float4(X.xxx,Balance)); }

// use default balance (can't do float4 version, would interfere): //

float stripe(float3 XYZ) {
    return stripe(float4(XYZ.xyz,DEFAULT_BALANCE)); }

float stripe(float2 XY) {
    return stripe(float4(XY.xyy,DEFAULT_BALANCE)); }

float stripe(float X) {
    return stripe(float4(X.xxx,DEFAULT_BALANCE)); }

///////////////////////////////////
// texture-free alternative ///////
///////////////////////////////////

float numeric_stripe(
	    float Value,
	    float Balance,
	    float Oversample,
	    float PatternScale
) {
    float width = abs(ddx(Value)) + abs(ddy(Value));
    float w = width*Oversample;
    float x0 = Value/PatternScale - (w/2.0);
    float x1 = x0 + w;
    float i0 = (1.0-Balance)*floor(x0) + max(0.0, frac(x0)-Balance);
    float i1 = (1.0-Balance)*floor(x1) + max(0.0, frac(x1)-Balance);
    float stripe = (i1 - i0)/w;
    stripe = min(1.0,max(0.0,stripe)); 
    return stripe;
}

///////////////////////////////////
// 2D checkerboard ////////////////
///////////////////////////////////

float checker2D(float4 XYZW)
{
    float stripex = tex2D(StripeSampler,XYZW.xw).x;
    float stripey = tex2D(StripeSampler,XYZW.yw).x;
    return abs(stripex - stripey);
}

// overloads of the above

float checker2D(float4 XYZW,float Balance) {
    return checker2D(float4(XYZW.xyz,Balance)); }

float checker2D(float3 XYZ,float Balance) {
    return checker2D(float4(XYZ.xyz,Balance)); }

float checker2D(float2 XY,float Balance) {
    return checker2D(float4(XY.xyy,Balance)); }

// use default balance ////////////////////////

float checker2D(float3 XYZ) {
    return checker2D(float4(XYZ.xyz,DEFAULT_BALANCE)); }

float checker2D(float2 XY) {
    return checker2D(float4(XY.xyy,DEFAULT_BALANCE)); }

float checker2D(float X) {
    return checker2D(float4(X.xxx,DEFAULT_BALANCE)); }

///////////////////////////////////
// 3D checkerboard ////////////////
///////////////////////////////////

float checker3D(float4 XYZW)
{
    float stripex = tex2D(StripeSampler,XYZW.xw).x;
    float stripey = tex2D(StripeSampler,XYZW.yw).x;
    float stripez = tex2D(StripeSampler,XYZW.zw).x;
    float check = abs(abs(stripex - stripey) - stripez);
    return check;
}

// overloads of the above

float checker3D(float3 XYZ,float Balance) {
    return checker3D(float4(XYZ.xyz,Balance)); }

float checker3D(float4 XYZW,float Balance) {
    return checker3D(float4(XYZW.xyz,Balance)); }

// use default balance ////////////////////////

float checker3D(float3 XYZ) {
    return checker3D(float4(XYZ.xyz,DEFAULT_BALANCE)); }

float checker3D(float2 XY) {
    return checker3D(float4(XY.xyy,DEFAULT_BALANCE)); }

float checker3D(float X) {
    return checker3D(float4(X.xxx,DEFAULT_BALANCE)); }

/////////////

float3 checker3Drgb(float4 XYZW)
{
    float3 result;
    result.x = tex2D(StripeSampler,XYZW.xw).x;
    result.y = tex2D(StripeSampler,XYZW.yw).x;
    result.z = tex2D(StripeSampler,XYZW.zw).x;
    return result;
}

float3 checker3Drgb(float3 XYZ,float Balance) {
    return checker3Drgb(float4(XYZ.xyz,Balance)); }

float3 checker3Drgb(float3 XYZ) {
    return checker3Drgb(float4(XYZ.xyz,DEFAULT_BALANCE)); }

#endif /* _H_STRIPE_TEX_ */


/******** TWEAKABLES ****************************************/

float3 Lamp0Dir : Direction = {0.7f,-0.7f,-0.7f};
float3 AmbiColor : Ambient = {0.07f, 0.07f, 0.07f};
float3 BrightColor = {1.0f, 0.8f, 0.3f};
float3 DarkColor = {0.0f, 0.2f, 0.4f};
float SpecExpon : SpecularPower = 20.0;
float Ks = 0.1;
float Balance = 0.1;
float Scale : UNITSSCALE = 0.1;

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
	float3 Position    : POSITION;
	float4 UV        : TEXCOORD0;
	float4 Normal    : NORMAL;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
	float4 HPosition    : POSITION;
	float4 TexCoord    : TEXCOORD0;
	float3 WorldNormal : TEXCOORD1;
	float3 WorldView : TEXCOORD2;
};

/*********** vertex shader ******/

vertexOutput mainVS(appdata IN) {
	vertexOutput OUT = (vertexOutput)0;
	float4 Po = float4(IN.Position.xyz,1.0); 
	float4 hpos  = mul(Po,matWorldViewProj);
	OUT.HPosition  = hpos;
	float4 Pw = mul(Po,matWorld);
	OUT.TexCoord = Pw * Scale;
	OUT.TexCoord.w = Balance;
	float4 Nn = normalize(IN.Normal);
	OUT.WorldView = normalize(matViewInv[3].xyz - Pw.xyz);
	OUT.WorldNormal = mul(Nn,matWorldInv).xyz;
	return OUT;
}

/******************** pixel shader *********************/

float4 strokeTexPS(vertexOutput IN) : COLOR {
	float stripex = tex2D(StripeSampler,float2(IN.TexCoord.xw)).x;
	float stripey = tex2D(StripeSampler,float2(IN.TexCoord.yw)).x;
	float stripez = tex2D(StripeSampler,float2(IN.TexCoord.zw)).x;
	float check = abs(abs(stripex - stripey) - stripez);
	float3 dColor = lerp(BrightColor,DarkColor,check);
	float3 Nn = normalize(IN.WorldNormal);
	float3 Vn = normalize(IN.WorldView);
	float3 Hn = normalize(Vn - Lamp0Dir);
	float4 litV = lit(dot(-Lamp0Dir,Nn),dot(Hn,Nn),SpecExpon);
	float spec = litV.z*check*Ks;
	float3 result = (dColor * (AmbiColor + litV.yyy)) + spec.xxx;
	return float4(result.rgb,1.0);
}

/*************/

technique checker3d
{
	pass p0
	{        
		VertexShader = compile vs_2_0 mainVS();
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		CullMode = None;
		PixelShader = compile ps_2_a strokeTexPS();
	}
}

/***************************** eof ***/
