float4x4 matWorldViewProj;
float4 vecTime;

texture entSkin1;
sampler basemap = sampler_state
{
	Texture = (entSkin1);
	ADDRESSU = clamp;
	ADDRESSV = clamp;
	ADDRESSW = clamp;
	MAGFILTER = LINEAR;
	MINFILTER = LINEAR;
	MIPFILTER = LINEAR;
};

texture mtlSkin1;
sampler noise1map = sampler_state
{
	Texture = (mtlSkin1);
	ADDRESSU = mirror;
	ADDRESSV = mirror;
	ADDRESSW = mirror;
};

texture mtlSkin2;
sampler noise2map = sampler_state
{
	Texture = (mtlSkin2);
	ADDRESSU = mirror;
	ADDRESSV = mirror;
	ADDRESSW = mirror;
};

texture mtlSkin3;
sampler colormap = sampler_state
{
	Texture = (mtlSkin3);
	ADDRESSU = wrap;
	ADDRESSV = wrap;
	ADDRESSW = wrap;
};

struct VS_IN
{
	float4 position : POSITION;
	float4 normal : NORMAL;
	float2 texCoord : TEXCOORD0;
};

struct VS_OUT
{
	float4 position : POSITION;
	float2 texCoord1 : TEXCOORD0;
	float2 texCoord2 : TEXCOORD1;
	float2 texCoord3 : TEXCOORD2;
	float2 texCoord4 : TEXCOORD3;
};

float waveHeight1 = 3.23;
float waveHeight2 = 4.1;
float waveScale1 = 23;
float waveScale2 = 23;
float waveSpeed1 = 23;
float waveSpeed2 = 23;

float texSpeed1 = 2723;
float texSpeed2 = 2933;
float texSpeed3 = 2841;
float texSpeed4 = 2623;

VS_OUT mainVS (VS_IN In , uniform float scale , uniform float speed)
{
	VS_OUT Out;
	In.position.xyz += In.normal * abs(waveHeight1 * sin(In.texCoord.x*waveScale1 + vecTime.w/waveSpeed1) + waveHeight2 * cos(In.texCoord.y*waveScale2 + vecTime.w/waveSpeed2));
	Out.position = mul(In.position, matWorldViewProj);
	Out.texCoord1 = In.texCoord;
	Out.texCoord2 = In.texCoord*scale;
	Out.texCoord3 = float2(In.position.y/500+vecTime.w/texSpeed1*speed,vecTime.w/texSpeed2*speed);
	Out.texCoord4 = -float2(In.position.z/500+vecTime.w/texSpeed3*speed,vecTime.w/texSpeed4*speed);
	return Out;
}

float4 color1= {2.0,0.0,0.0,1.0};

float4 lavaPS (VS_OUT In) : COLOR0
{
	float4 base = tex2D(basemap, In.texCoord1);
	float4 noise1 = tex2D(noise1map, In.texCoord2);
	float4 noise2 = tex2D(noise2map, In.texCoord2+In.texCoord3);
	float4 noise3 = tex2D(noise2map, In.texCoord2+In.texCoord4);
	float4 lava1 = tex1D(colormap,lerp(noise3.r,noise2.r,noise2.g));
	float4 lava2 = tex1D(colormap,lerp(noise3.g,noise2.b,noise3.b));
	float4 result = (base + (lava1*lava2 + color1*noise2.b))/2;
	return result;
}


float4 color2 = {2.0,1.0,0.0,1.0};
float4 color3 = {2.0,1.5,0.0,1.0};
float4 color4 = {3.0,2.0,0.0,1.0};

float4 lava1PS (VS_OUT In) : COLOR0
{
	float4 noise1 = tex2D(noise1map, In.texCoord2);
	float4 noise2 = tex2D(noise2map, In.texCoord2+In.texCoord3);
	float4 noise3 = tex2D(noise2map, In.texCoord2+In.texCoord4);
	float4 result = noise1.r * color2 + noise2.g * color3 + noise3.b * color4;
	return result / 3;
}


float4 lava2PS (VS_OUT In) : COLOR0
{
	float4 noise1 = tex2D(noise1map, In.texCoord2);
	float4 noise2 = tex2D(noise2map, In.texCoord2+In.texCoord3);
	float4 noise3 = tex2D(noise2map, In.texCoord2+In.texCoord4);
	float4 result = noise3.r * color2 + noise2.r * color3 + noise3.b * color4;
	return result / 3;
}

float4 lava3PS (VS_OUT In) : COLOR0
{
	float4 noise1 = tex2D(noise1map, In.texCoord2);
	float4 noise2 = tex2D(noise2map, In.texCoord2+In.texCoord3);
	float4 noise3 = tex2D(noise2map, In.texCoord2+In.texCoord4);
	float4 result = noise1.r * color2 + noise2.g * color3 + noise1.b * color4;
	return result / 3;
}

technique lava
{
	pass p0
	{
		VertexShader = compile vs_2_0 mainVS(0.65,1);
		PixelShader = compile ps_2_0 lavaPS();
	}
	pass p1
	{
		AlphaBlendEnable = 1;
		blendOp = 1;
		srcBlend = 3;
		destBlend = 9;
		VertexShader = compile vs_2_0 mainVS(1.35,2);
		PixelShader = compile ps_2_0 lava1PS();
	}
	pass p2
	{
		VertexShader = compile vs_2_0 mainVS(3.35,0.23);
		PixelShader = compile ps_2_0 lava2PS();
	}
	pass p3
	{
		srcBlend = 9;
		destBlend = 5;
		VertexShader = compile vs_2_0 mainVS(2.35,1.53);
		PixelShader = compile ps_2_0 lava3PS();
	}
}